# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ComputeInstanceArgs', 'ComputeInstance']

@pulumi.input_type
class ComputeInstanceArgs:
    def __init__(__self__, *,
                 template_id: pulumi.Input[str],
                 type: pulumi.Input[str],
                 zone: pulumi.Input[str],
                 anti_affinity_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 block_storage_volume_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 deploy_target_id: Optional[pulumi.Input[str]] = None,
                 destroy_protected: Optional[pulumi.Input[bool]] = None,
                 disk_size: Optional[pulumi.Input[int]] = None,
                 elastic_ip_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ipv6: Optional[pulumi.Input[bool]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input['ComputeInstanceNetworkInterfaceArgs']]]] = None,
                 private: Optional[pulumi.Input[bool]] = None,
                 reverse_dns: Optional[pulumi.Input[str]] = None,
                 security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ssh_key: Optional[pulumi.Input[str]] = None,
                 state: Optional[pulumi.Input[str]] = None,
                 user_data: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a ComputeInstance resource.
        :param pulumi.Input[str] template_id: ❗ The get_template (ID) to use when creating the instance.
        :param pulumi.Input[str] type: The instance type (`<family>.<size>`, e.g. `standard.medium`; use the [Exoscale CLI](https://github.com/exoscale/cli/) - `exo compute instance-type list` - for the list of available types). **WARNING**: updating this attribute stops/restarts the instance.
        :param pulumi.Input[str] zone: ❗ The Exoscale [Zone](https://www.exoscale.com/datacenters/) name.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] anti_affinity_group_ids: ❗ A list of exoscale*anti*affinity_group (IDs) to attach to the instance (may only be set at creation time).
        :param pulumi.Input[Sequence[pulumi.Input[str]]] block_storage_volume_ids: A list of exoscale*block*storage_volume (ID) to attach to the instance.
        :param pulumi.Input[str] deploy_target_id: ❗ A deploy target ID.
        :param pulumi.Input[bool] destroy_protected: Mark the instance as protected, the Exoscale API will refuse to delete the instance until the protection is removed (boolean; default: `false`).
        :param pulumi.Input[int] disk_size: The instance disk size (GiB; at least `10`). Can not be decreased after creation. **WARNING**: updating this attribute stops/restarts the instance.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] elastic_ip_ids: A list of exoscale*elastic*ip (IDs) to attach to the instance.
        :param pulumi.Input[bool] ipv6: Enable IPv6 on the instance (boolean; default: `false`).
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: A map of key/value labels.
        :param pulumi.Input[str] name: The compute instance name.
        :param pulumi.Input[Sequence[pulumi.Input['ComputeInstanceNetworkInterfaceArgs']]] network_interfaces: Private network interfaces (may be specified multiple times). Structure is documented below.
        :param pulumi.Input[bool] private: Whether the instance is private (no public IP addresses; default: false)
        :param pulumi.Input[str] reverse_dns: Domain name for reverse DNS record.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] security_group_ids: A list of exoscale*security*group (IDs) to attach to the instance.
        :param pulumi.Input[str] ssh_key: The exoscale*ssh*key (name) to authorize in the instance (may only be set at creation time).
        :param pulumi.Input[str] state: The instance state (`running` or `stopped`; default: `running`).
        :param pulumi.Input[str] user_data: [cloud-init](https://cloudinit.readthedocs.io/) configuration.
        """
        pulumi.set(__self__, "template_id", template_id)
        pulumi.set(__self__, "type", type)
        pulumi.set(__self__, "zone", zone)
        if anti_affinity_group_ids is not None:
            pulumi.set(__self__, "anti_affinity_group_ids", anti_affinity_group_ids)
        if block_storage_volume_ids is not None:
            pulumi.set(__self__, "block_storage_volume_ids", block_storage_volume_ids)
        if deploy_target_id is not None:
            pulumi.set(__self__, "deploy_target_id", deploy_target_id)
        if destroy_protected is not None:
            pulumi.set(__self__, "destroy_protected", destroy_protected)
        if disk_size is not None:
            pulumi.set(__self__, "disk_size", disk_size)
        if elastic_ip_ids is not None:
            pulumi.set(__self__, "elastic_ip_ids", elastic_ip_ids)
        if ipv6 is not None:
            pulumi.set(__self__, "ipv6", ipv6)
        if labels is not None:
            pulumi.set(__self__, "labels", labels)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if network_interfaces is not None:
            pulumi.set(__self__, "network_interfaces", network_interfaces)
        if private is not None:
            pulumi.set(__self__, "private", private)
        if reverse_dns is not None:
            pulumi.set(__self__, "reverse_dns", reverse_dns)
        if security_group_ids is not None:
            pulumi.set(__self__, "security_group_ids", security_group_ids)
        if ssh_key is not None:
            pulumi.set(__self__, "ssh_key", ssh_key)
        if state is not None:
            pulumi.set(__self__, "state", state)
        if user_data is not None:
            pulumi.set(__self__, "user_data", user_data)

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> pulumi.Input[str]:
        """
        ❗ The get_template (ID) to use when creating the instance.
        """
        return pulumi.get(self, "template_id")

    @template_id.setter
    def template_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "template_id", value)

    @property
    @pulumi.getter
    def type(self) -> pulumi.Input[str]:
        """
        The instance type (`<family>.<size>`, e.g. `standard.medium`; use the [Exoscale CLI](https://github.com/exoscale/cli/) - `exo compute instance-type list` - for the list of available types). **WARNING**: updating this attribute stops/restarts the instance.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: pulumi.Input[str]):
        pulumi.set(self, "type", value)

    @property
    @pulumi.getter
    def zone(self) -> pulumi.Input[str]:
        """
        ❗ The Exoscale [Zone](https://www.exoscale.com/datacenters/) name.
        """
        return pulumi.get(self, "zone")

    @zone.setter
    def zone(self, value: pulumi.Input[str]):
        pulumi.set(self, "zone", value)

    @property
    @pulumi.getter(name="antiAffinityGroupIds")
    def anti_affinity_group_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        ❗ A list of exoscale*anti*affinity_group (IDs) to attach to the instance (may only be set at creation time).
        """
        return pulumi.get(self, "anti_affinity_group_ids")

    @anti_affinity_group_ids.setter
    def anti_affinity_group_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "anti_affinity_group_ids", value)

    @property
    @pulumi.getter(name="blockStorageVolumeIds")
    def block_storage_volume_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of exoscale*block*storage_volume (ID) to attach to the instance.
        """
        return pulumi.get(self, "block_storage_volume_ids")

    @block_storage_volume_ids.setter
    def block_storage_volume_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "block_storage_volume_ids", value)

    @property
    @pulumi.getter(name="deployTargetId")
    def deploy_target_id(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ A deploy target ID.
        """
        return pulumi.get(self, "deploy_target_id")

    @deploy_target_id.setter
    def deploy_target_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "deploy_target_id", value)

    @property
    @pulumi.getter(name="destroyProtected")
    def destroy_protected(self) -> Optional[pulumi.Input[bool]]:
        """
        Mark the instance as protected, the Exoscale API will refuse to delete the instance until the protection is removed (boolean; default: `false`).
        """
        return pulumi.get(self, "destroy_protected")

    @destroy_protected.setter
    def destroy_protected(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "destroy_protected", value)

    @property
    @pulumi.getter(name="diskSize")
    def disk_size(self) -> Optional[pulumi.Input[int]]:
        """
        The instance disk size (GiB; at least `10`). Can not be decreased after creation. **WARNING**: updating this attribute stops/restarts the instance.
        """
        return pulumi.get(self, "disk_size")

    @disk_size.setter
    def disk_size(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "disk_size", value)

    @property
    @pulumi.getter(name="elasticIpIds")
    def elastic_ip_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of exoscale*elastic*ip (IDs) to attach to the instance.
        """
        return pulumi.get(self, "elastic_ip_ids")

    @elastic_ip_ids.setter
    def elastic_ip_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "elastic_ip_ids", value)

    @property
    @pulumi.getter
    def ipv6(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable IPv6 on the instance (boolean; default: `false`).
        """
        return pulumi.get(self, "ipv6")

    @ipv6.setter
    def ipv6(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "ipv6", value)

    @property
    @pulumi.getter
    def labels(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        A map of key/value labels.
        """
        return pulumi.get(self, "labels")

    @labels.setter
    def labels(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "labels", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The compute instance name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="networkInterfaces")
    def network_interfaces(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ComputeInstanceNetworkInterfaceArgs']]]]:
        """
        Private network interfaces (may be specified multiple times). Structure is documented below.
        """
        return pulumi.get(self, "network_interfaces")

    @network_interfaces.setter
    def network_interfaces(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ComputeInstanceNetworkInterfaceArgs']]]]):
        pulumi.set(self, "network_interfaces", value)

    @property
    @pulumi.getter
    def private(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether the instance is private (no public IP addresses; default: false)
        """
        return pulumi.get(self, "private")

    @private.setter
    def private(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "private", value)

    @property
    @pulumi.getter(name="reverseDns")
    def reverse_dns(self) -> Optional[pulumi.Input[str]]:
        """
        Domain name for reverse DNS record.
        """
        return pulumi.get(self, "reverse_dns")

    @reverse_dns.setter
    def reverse_dns(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "reverse_dns", value)

    @property
    @pulumi.getter(name="securityGroupIds")
    def security_group_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of exoscale*security*group (IDs) to attach to the instance.
        """
        return pulumi.get(self, "security_group_ids")

    @security_group_ids.setter
    def security_group_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "security_group_ids", value)

    @property
    @pulumi.getter(name="sshKey")
    def ssh_key(self) -> Optional[pulumi.Input[str]]:
        """
        The exoscale*ssh*key (name) to authorize in the instance (may only be set at creation time).
        """
        return pulumi.get(self, "ssh_key")

    @ssh_key.setter
    def ssh_key(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ssh_key", value)

    @property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[str]]:
        """
        The instance state (`running` or `stopped`; default: `running`).
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "state", value)

    @property
    @pulumi.getter(name="userData")
    def user_data(self) -> Optional[pulumi.Input[str]]:
        """
        [cloud-init](https://cloudinit.readthedocs.io/) configuration.
        """
        return pulumi.get(self, "user_data")

    @user_data.setter
    def user_data(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_data", value)


@pulumi.input_type
class _ComputeInstanceState:
    def __init__(__self__, *,
                 anti_affinity_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 block_storage_volume_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 created_at: Optional[pulumi.Input[str]] = None,
                 deploy_target_id: Optional[pulumi.Input[str]] = None,
                 destroy_protected: Optional[pulumi.Input[bool]] = None,
                 disk_size: Optional[pulumi.Input[int]] = None,
                 elastic_ip_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ipv6: Optional[pulumi.Input[bool]] = None,
                 ipv6_address: Optional[pulumi.Input[str]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 mac_address: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input['ComputeInstanceNetworkInterfaceArgs']]]] = None,
                 private: Optional[pulumi.Input[bool]] = None,
                 private_network_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 public_ip_address: Optional[pulumi.Input[str]] = None,
                 reverse_dns: Optional[pulumi.Input[str]] = None,
                 security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ssh_key: Optional[pulumi.Input[str]] = None,
                 state: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 user_data: Optional[pulumi.Input[str]] = None,
                 zone: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering ComputeInstance resources.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] anti_affinity_group_ids: ❗ A list of exoscale*anti*affinity_group (IDs) to attach to the instance (may only be set at creation time).
        :param pulumi.Input[Sequence[pulumi.Input[str]]] block_storage_volume_ids: A list of exoscale*block*storage_volume (ID) to attach to the instance.
        :param pulumi.Input[str] created_at: The instance creation date.
        :param pulumi.Input[str] deploy_target_id: ❗ A deploy target ID.
        :param pulumi.Input[bool] destroy_protected: Mark the instance as protected, the Exoscale API will refuse to delete the instance until the protection is removed (boolean; default: `false`).
        :param pulumi.Input[int] disk_size: The instance disk size (GiB; at least `10`). Can not be decreased after creation. **WARNING**: updating this attribute stops/restarts the instance.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] elastic_ip_ids: A list of exoscale*elastic*ip (IDs) to attach to the instance.
        :param pulumi.Input[bool] ipv6: Enable IPv6 on the instance (boolean; default: `false`).
        :param pulumi.Input[str] ipv6_address: The instance (main network interface) IPv6 address (if enabled).
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: A map of key/value labels.
        :param pulumi.Input[str] mac_address: MAC address
        :param pulumi.Input[str] name: The compute instance name.
        :param pulumi.Input[Sequence[pulumi.Input['ComputeInstanceNetworkInterfaceArgs']]] network_interfaces: Private network interfaces (may be specified multiple times). Structure is documented below.
        :param pulumi.Input[bool] private: Whether the instance is private (no public IP addresses; default: false)
        :param pulumi.Input[Sequence[pulumi.Input[str]]] private_network_ids: A list of private networks (IDs) attached to the instance. Please use the `network_interface.*.network_id` argument instead.
        :param pulumi.Input[str] public_ip_address: The instance (main network interface) IPv4 address.
        :param pulumi.Input[str] reverse_dns: Domain name for reverse DNS record.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] security_group_ids: A list of exoscale*security*group (IDs) to attach to the instance.
        :param pulumi.Input[str] ssh_key: The exoscale*ssh*key (name) to authorize in the instance (may only be set at creation time).
        :param pulumi.Input[str] state: The instance state (`running` or `stopped`; default: `running`).
        :param pulumi.Input[str] template_id: ❗ The get_template (ID) to use when creating the instance.
        :param pulumi.Input[str] type: The instance type (`<family>.<size>`, e.g. `standard.medium`; use the [Exoscale CLI](https://github.com/exoscale/cli/) - `exo compute instance-type list` - for the list of available types). **WARNING**: updating this attribute stops/restarts the instance.
        :param pulumi.Input[str] user_data: [cloud-init](https://cloudinit.readthedocs.io/) configuration.
        :param pulumi.Input[str] zone: ❗ The Exoscale [Zone](https://www.exoscale.com/datacenters/) name.
        """
        if anti_affinity_group_ids is not None:
            pulumi.set(__self__, "anti_affinity_group_ids", anti_affinity_group_ids)
        if block_storage_volume_ids is not None:
            pulumi.set(__self__, "block_storage_volume_ids", block_storage_volume_ids)
        if created_at is not None:
            pulumi.set(__self__, "created_at", created_at)
        if deploy_target_id is not None:
            pulumi.set(__self__, "deploy_target_id", deploy_target_id)
        if destroy_protected is not None:
            pulumi.set(__self__, "destroy_protected", destroy_protected)
        if disk_size is not None:
            pulumi.set(__self__, "disk_size", disk_size)
        if elastic_ip_ids is not None:
            pulumi.set(__self__, "elastic_ip_ids", elastic_ip_ids)
        if ipv6 is not None:
            pulumi.set(__self__, "ipv6", ipv6)
        if ipv6_address is not None:
            pulumi.set(__self__, "ipv6_address", ipv6_address)
        if labels is not None:
            pulumi.set(__self__, "labels", labels)
        if mac_address is not None:
            pulumi.set(__self__, "mac_address", mac_address)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if network_interfaces is not None:
            pulumi.set(__self__, "network_interfaces", network_interfaces)
        if private is not None:
            pulumi.set(__self__, "private", private)
        if private_network_ids is not None:
            warnings.warn("""Use the network_interface block instead.""", DeprecationWarning)
            pulumi.log.warn("""private_network_ids is deprecated: Use the network_interface block instead.""")
        if private_network_ids is not None:
            pulumi.set(__self__, "private_network_ids", private_network_ids)
        if public_ip_address is not None:
            pulumi.set(__self__, "public_ip_address", public_ip_address)
        if reverse_dns is not None:
            pulumi.set(__self__, "reverse_dns", reverse_dns)
        if security_group_ids is not None:
            pulumi.set(__self__, "security_group_ids", security_group_ids)
        if ssh_key is not None:
            pulumi.set(__self__, "ssh_key", ssh_key)
        if state is not None:
            pulumi.set(__self__, "state", state)
        if template_id is not None:
            pulumi.set(__self__, "template_id", template_id)
        if type is not None:
            pulumi.set(__self__, "type", type)
        if user_data is not None:
            pulumi.set(__self__, "user_data", user_data)
        if zone is not None:
            pulumi.set(__self__, "zone", zone)

    @property
    @pulumi.getter(name="antiAffinityGroupIds")
    def anti_affinity_group_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        ❗ A list of exoscale*anti*affinity_group (IDs) to attach to the instance (may only be set at creation time).
        """
        return pulumi.get(self, "anti_affinity_group_ids")

    @anti_affinity_group_ids.setter
    def anti_affinity_group_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "anti_affinity_group_ids", value)

    @property
    @pulumi.getter(name="blockStorageVolumeIds")
    def block_storage_volume_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of exoscale*block*storage_volume (ID) to attach to the instance.
        """
        return pulumi.get(self, "block_storage_volume_ids")

    @block_storage_volume_ids.setter
    def block_storage_volume_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "block_storage_volume_ids", value)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> Optional[pulumi.Input[str]]:
        """
        The instance creation date.
        """
        return pulumi.get(self, "created_at")

    @created_at.setter
    def created_at(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "created_at", value)

    @property
    @pulumi.getter(name="deployTargetId")
    def deploy_target_id(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ A deploy target ID.
        """
        return pulumi.get(self, "deploy_target_id")

    @deploy_target_id.setter
    def deploy_target_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "deploy_target_id", value)

    @property
    @pulumi.getter(name="destroyProtected")
    def destroy_protected(self) -> Optional[pulumi.Input[bool]]:
        """
        Mark the instance as protected, the Exoscale API will refuse to delete the instance until the protection is removed (boolean; default: `false`).
        """
        return pulumi.get(self, "destroy_protected")

    @destroy_protected.setter
    def destroy_protected(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "destroy_protected", value)

    @property
    @pulumi.getter(name="diskSize")
    def disk_size(self) -> Optional[pulumi.Input[int]]:
        """
        The instance disk size (GiB; at least `10`). Can not be decreased after creation. **WARNING**: updating this attribute stops/restarts the instance.
        """
        return pulumi.get(self, "disk_size")

    @disk_size.setter
    def disk_size(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "disk_size", value)

    @property
    @pulumi.getter(name="elasticIpIds")
    def elastic_ip_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of exoscale*elastic*ip (IDs) to attach to the instance.
        """
        return pulumi.get(self, "elastic_ip_ids")

    @elastic_ip_ids.setter
    def elastic_ip_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "elastic_ip_ids", value)

    @property
    @pulumi.getter
    def ipv6(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable IPv6 on the instance (boolean; default: `false`).
        """
        return pulumi.get(self, "ipv6")

    @ipv6.setter
    def ipv6(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "ipv6", value)

    @property
    @pulumi.getter(name="ipv6Address")
    def ipv6_address(self) -> Optional[pulumi.Input[str]]:
        """
        The instance (main network interface) IPv6 address (if enabled).
        """
        return pulumi.get(self, "ipv6_address")

    @ipv6_address.setter
    def ipv6_address(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv6_address", value)

    @property
    @pulumi.getter
    def labels(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        A map of key/value labels.
        """
        return pulumi.get(self, "labels")

    @labels.setter
    def labels(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "labels", value)

    @property
    @pulumi.getter(name="macAddress")
    def mac_address(self) -> Optional[pulumi.Input[str]]:
        """
        MAC address
        """
        return pulumi.get(self, "mac_address")

    @mac_address.setter
    def mac_address(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mac_address", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The compute instance name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="networkInterfaces")
    def network_interfaces(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ComputeInstanceNetworkInterfaceArgs']]]]:
        """
        Private network interfaces (may be specified multiple times). Structure is documented below.
        """
        return pulumi.get(self, "network_interfaces")

    @network_interfaces.setter
    def network_interfaces(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ComputeInstanceNetworkInterfaceArgs']]]]):
        pulumi.set(self, "network_interfaces", value)

    @property
    @pulumi.getter
    def private(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether the instance is private (no public IP addresses; default: false)
        """
        return pulumi.get(self, "private")

    @private.setter
    def private(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "private", value)

    @property
    @pulumi.getter(name="privateNetworkIds")
    @_utilities.deprecated("""Use the network_interface block instead.""")
    def private_network_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of private networks (IDs) attached to the instance. Please use the `network_interface.*.network_id` argument instead.
        """
        return pulumi.get(self, "private_network_ids")

    @private_network_ids.setter
    def private_network_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "private_network_ids", value)

    @property
    @pulumi.getter(name="publicIpAddress")
    def public_ip_address(self) -> Optional[pulumi.Input[str]]:
        """
        The instance (main network interface) IPv4 address.
        """
        return pulumi.get(self, "public_ip_address")

    @public_ip_address.setter
    def public_ip_address(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "public_ip_address", value)

    @property
    @pulumi.getter(name="reverseDns")
    def reverse_dns(self) -> Optional[pulumi.Input[str]]:
        """
        Domain name for reverse DNS record.
        """
        return pulumi.get(self, "reverse_dns")

    @reverse_dns.setter
    def reverse_dns(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "reverse_dns", value)

    @property
    @pulumi.getter(name="securityGroupIds")
    def security_group_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of exoscale*security*group (IDs) to attach to the instance.
        """
        return pulumi.get(self, "security_group_ids")

    @security_group_ids.setter
    def security_group_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "security_group_ids", value)

    @property
    @pulumi.getter(name="sshKey")
    def ssh_key(self) -> Optional[pulumi.Input[str]]:
        """
        The exoscale*ssh*key (name) to authorize in the instance (may only be set at creation time).
        """
        return pulumi.get(self, "ssh_key")

    @ssh_key.setter
    def ssh_key(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ssh_key", value)

    @property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[str]]:
        """
        The instance state (`running` or `stopped`; default: `running`).
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "state", value)

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The get_template (ID) to use when creating the instance.
        """
        return pulumi.get(self, "template_id")

    @template_id.setter
    def template_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "template_id", value)

    @property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[str]]:
        """
        The instance type (`<family>.<size>`, e.g. `standard.medium`; use the [Exoscale CLI](https://github.com/exoscale/cli/) - `exo compute instance-type list` - for the list of available types). **WARNING**: updating this attribute stops/restarts the instance.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "type", value)

    @property
    @pulumi.getter(name="userData")
    def user_data(self) -> Optional[pulumi.Input[str]]:
        """
        [cloud-init](https://cloudinit.readthedocs.io/) configuration.
        """
        return pulumi.get(self, "user_data")

    @user_data.setter
    def user_data(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_data", value)

    @property
    @pulumi.getter
    def zone(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The Exoscale [Zone](https://www.exoscale.com/datacenters/) name.
        """
        return pulumi.get(self, "zone")

    @zone.setter
    def zone(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "zone", value)


class ComputeInstance(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 anti_affinity_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 block_storage_volume_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 deploy_target_id: Optional[pulumi.Input[str]] = None,
                 destroy_protected: Optional[pulumi.Input[bool]] = None,
                 disk_size: Optional[pulumi.Input[int]] = None,
                 elastic_ip_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ipv6: Optional[pulumi.Input[bool]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ComputeInstanceNetworkInterfaceArgs', 'ComputeInstanceNetworkInterfaceArgsDict']]]]] = None,
                 private: Optional[pulumi.Input[bool]] = None,
                 reverse_dns: Optional[pulumi.Input[str]] = None,
                 security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ssh_key: Optional[pulumi.Input[str]] = None,
                 state: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 user_data: Optional[pulumi.Input[str]] = None,
                 zone: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manage Exoscale [Compute Instances](https://community.exoscale.com/documentation/compute/).

        Corresponding data sources: exoscale_compute_instance, exoscale_compute_instance_list.

        After the creation, you can retrieve the password of an instance with [Exoscale CLI](https://github.com/exoscale/cli): `exo compute instance reveal-password NAME`.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_exoscale as exoscale
        import pulumiverse_exoscale as exoscale

        my_template = exoscale.get_template(zone="ch-gva-2",
            name="Linux Ubuntu 22.04 LTS 64-bit")
        my_instance = exoscale.ComputeInstance("myInstance",
            zone="ch-gva-2",
            template_id=my_template.id,
            type="standard.medium",
            disk_size=10)
        ```

        Please refer to the examples
        directory for complete configuration examples.

        ## Import

        An existing compute instance may be imported by `<ID>@<zone>`:

        ```sh
        $ pulumi import exoscale:index/computeInstance:ComputeInstance \\
        ```

          exoscale_compute_instance.my_instance \\

          f81d4fae-7dec-11d0-a765-00a0c91e6bf6@ch-gva-2

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] anti_affinity_group_ids: ❗ A list of exoscale*anti*affinity_group (IDs) to attach to the instance (may only be set at creation time).
        :param pulumi.Input[Sequence[pulumi.Input[str]]] block_storage_volume_ids: A list of exoscale*block*storage_volume (ID) to attach to the instance.
        :param pulumi.Input[str] deploy_target_id: ❗ A deploy target ID.
        :param pulumi.Input[bool] destroy_protected: Mark the instance as protected, the Exoscale API will refuse to delete the instance until the protection is removed (boolean; default: `false`).
        :param pulumi.Input[int] disk_size: The instance disk size (GiB; at least `10`). Can not be decreased after creation. **WARNING**: updating this attribute stops/restarts the instance.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] elastic_ip_ids: A list of exoscale*elastic*ip (IDs) to attach to the instance.
        :param pulumi.Input[bool] ipv6: Enable IPv6 on the instance (boolean; default: `false`).
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: A map of key/value labels.
        :param pulumi.Input[str] name: The compute instance name.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ComputeInstanceNetworkInterfaceArgs', 'ComputeInstanceNetworkInterfaceArgsDict']]]] network_interfaces: Private network interfaces (may be specified multiple times). Structure is documented below.
        :param pulumi.Input[bool] private: Whether the instance is private (no public IP addresses; default: false)
        :param pulumi.Input[str] reverse_dns: Domain name for reverse DNS record.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] security_group_ids: A list of exoscale*security*group (IDs) to attach to the instance.
        :param pulumi.Input[str] ssh_key: The exoscale*ssh*key (name) to authorize in the instance (may only be set at creation time).
        :param pulumi.Input[str] state: The instance state (`running` or `stopped`; default: `running`).
        :param pulumi.Input[str] template_id: ❗ The get_template (ID) to use when creating the instance.
        :param pulumi.Input[str] type: The instance type (`<family>.<size>`, e.g. `standard.medium`; use the [Exoscale CLI](https://github.com/exoscale/cli/) - `exo compute instance-type list` - for the list of available types). **WARNING**: updating this attribute stops/restarts the instance.
        :param pulumi.Input[str] user_data: [cloud-init](https://cloudinit.readthedocs.io/) configuration.
        :param pulumi.Input[str] zone: ❗ The Exoscale [Zone](https://www.exoscale.com/datacenters/) name.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ComputeInstanceArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manage Exoscale [Compute Instances](https://community.exoscale.com/documentation/compute/).

        Corresponding data sources: exoscale_compute_instance, exoscale_compute_instance_list.

        After the creation, you can retrieve the password of an instance with [Exoscale CLI](https://github.com/exoscale/cli): `exo compute instance reveal-password NAME`.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_exoscale as exoscale
        import pulumiverse_exoscale as exoscale

        my_template = exoscale.get_template(zone="ch-gva-2",
            name="Linux Ubuntu 22.04 LTS 64-bit")
        my_instance = exoscale.ComputeInstance("myInstance",
            zone="ch-gva-2",
            template_id=my_template.id,
            type="standard.medium",
            disk_size=10)
        ```

        Please refer to the examples
        directory for complete configuration examples.

        ## Import

        An existing compute instance may be imported by `<ID>@<zone>`:

        ```sh
        $ pulumi import exoscale:index/computeInstance:ComputeInstance \\
        ```

          exoscale_compute_instance.my_instance \\

          f81d4fae-7dec-11d0-a765-00a0c91e6bf6@ch-gva-2

        :param str resource_name: The name of the resource.
        :param ComputeInstanceArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ComputeInstanceArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 anti_affinity_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 block_storage_volume_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 deploy_target_id: Optional[pulumi.Input[str]] = None,
                 destroy_protected: Optional[pulumi.Input[bool]] = None,
                 disk_size: Optional[pulumi.Input[int]] = None,
                 elastic_ip_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ipv6: Optional[pulumi.Input[bool]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ComputeInstanceNetworkInterfaceArgs', 'ComputeInstanceNetworkInterfaceArgsDict']]]]] = None,
                 private: Optional[pulumi.Input[bool]] = None,
                 reverse_dns: Optional[pulumi.Input[str]] = None,
                 security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ssh_key: Optional[pulumi.Input[str]] = None,
                 state: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 user_data: Optional[pulumi.Input[str]] = None,
                 zone: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ComputeInstanceArgs.__new__(ComputeInstanceArgs)

            __props__.__dict__["anti_affinity_group_ids"] = anti_affinity_group_ids
            __props__.__dict__["block_storage_volume_ids"] = block_storage_volume_ids
            __props__.__dict__["deploy_target_id"] = deploy_target_id
            __props__.__dict__["destroy_protected"] = destroy_protected
            __props__.__dict__["disk_size"] = disk_size
            __props__.__dict__["elastic_ip_ids"] = elastic_ip_ids
            __props__.__dict__["ipv6"] = ipv6
            __props__.__dict__["labels"] = labels
            __props__.__dict__["name"] = name
            __props__.__dict__["network_interfaces"] = network_interfaces
            __props__.__dict__["private"] = private
            __props__.__dict__["reverse_dns"] = reverse_dns
            __props__.__dict__["security_group_ids"] = security_group_ids
            __props__.__dict__["ssh_key"] = ssh_key
            __props__.__dict__["state"] = state
            if template_id is None and not opts.urn:
                raise TypeError("Missing required property 'template_id'")
            __props__.__dict__["template_id"] = template_id
            if type is None and not opts.urn:
                raise TypeError("Missing required property 'type'")
            __props__.__dict__["type"] = type
            __props__.__dict__["user_data"] = user_data
            if zone is None and not opts.urn:
                raise TypeError("Missing required property 'zone'")
            __props__.__dict__["zone"] = zone
            __props__.__dict__["created_at"] = None
            __props__.__dict__["ipv6_address"] = None
            __props__.__dict__["mac_address"] = None
            __props__.__dict__["private_network_ids"] = None
            __props__.__dict__["public_ip_address"] = None
        super(ComputeInstance, __self__).__init__(
            'exoscale:index/computeInstance:ComputeInstance',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            anti_affinity_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            block_storage_volume_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            created_at: Optional[pulumi.Input[str]] = None,
            deploy_target_id: Optional[pulumi.Input[str]] = None,
            destroy_protected: Optional[pulumi.Input[bool]] = None,
            disk_size: Optional[pulumi.Input[int]] = None,
            elastic_ip_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            ipv6: Optional[pulumi.Input[bool]] = None,
            ipv6_address: Optional[pulumi.Input[str]] = None,
            labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            mac_address: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ComputeInstanceNetworkInterfaceArgs', 'ComputeInstanceNetworkInterfaceArgsDict']]]]] = None,
            private: Optional[pulumi.Input[bool]] = None,
            private_network_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            public_ip_address: Optional[pulumi.Input[str]] = None,
            reverse_dns: Optional[pulumi.Input[str]] = None,
            security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            ssh_key: Optional[pulumi.Input[str]] = None,
            state: Optional[pulumi.Input[str]] = None,
            template_id: Optional[pulumi.Input[str]] = None,
            type: Optional[pulumi.Input[str]] = None,
            user_data: Optional[pulumi.Input[str]] = None,
            zone: Optional[pulumi.Input[str]] = None) -> 'ComputeInstance':
        """
        Get an existing ComputeInstance resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] anti_affinity_group_ids: ❗ A list of exoscale*anti*affinity_group (IDs) to attach to the instance (may only be set at creation time).
        :param pulumi.Input[Sequence[pulumi.Input[str]]] block_storage_volume_ids: A list of exoscale*block*storage_volume (ID) to attach to the instance.
        :param pulumi.Input[str] created_at: The instance creation date.
        :param pulumi.Input[str] deploy_target_id: ❗ A deploy target ID.
        :param pulumi.Input[bool] destroy_protected: Mark the instance as protected, the Exoscale API will refuse to delete the instance until the protection is removed (boolean; default: `false`).
        :param pulumi.Input[int] disk_size: The instance disk size (GiB; at least `10`). Can not be decreased after creation. **WARNING**: updating this attribute stops/restarts the instance.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] elastic_ip_ids: A list of exoscale*elastic*ip (IDs) to attach to the instance.
        :param pulumi.Input[bool] ipv6: Enable IPv6 on the instance (boolean; default: `false`).
        :param pulumi.Input[str] ipv6_address: The instance (main network interface) IPv6 address (if enabled).
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: A map of key/value labels.
        :param pulumi.Input[str] mac_address: MAC address
        :param pulumi.Input[str] name: The compute instance name.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ComputeInstanceNetworkInterfaceArgs', 'ComputeInstanceNetworkInterfaceArgsDict']]]] network_interfaces: Private network interfaces (may be specified multiple times). Structure is documented below.
        :param pulumi.Input[bool] private: Whether the instance is private (no public IP addresses; default: false)
        :param pulumi.Input[Sequence[pulumi.Input[str]]] private_network_ids: A list of private networks (IDs) attached to the instance. Please use the `network_interface.*.network_id` argument instead.
        :param pulumi.Input[str] public_ip_address: The instance (main network interface) IPv4 address.
        :param pulumi.Input[str] reverse_dns: Domain name for reverse DNS record.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] security_group_ids: A list of exoscale*security*group (IDs) to attach to the instance.
        :param pulumi.Input[str] ssh_key: The exoscale*ssh*key (name) to authorize in the instance (may only be set at creation time).
        :param pulumi.Input[str] state: The instance state (`running` or `stopped`; default: `running`).
        :param pulumi.Input[str] template_id: ❗ The get_template (ID) to use when creating the instance.
        :param pulumi.Input[str] type: The instance type (`<family>.<size>`, e.g. `standard.medium`; use the [Exoscale CLI](https://github.com/exoscale/cli/) - `exo compute instance-type list` - for the list of available types). **WARNING**: updating this attribute stops/restarts the instance.
        :param pulumi.Input[str] user_data: [cloud-init](https://cloudinit.readthedocs.io/) configuration.
        :param pulumi.Input[str] zone: ❗ The Exoscale [Zone](https://www.exoscale.com/datacenters/) name.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ComputeInstanceState.__new__(_ComputeInstanceState)

        __props__.__dict__["anti_affinity_group_ids"] = anti_affinity_group_ids
        __props__.__dict__["block_storage_volume_ids"] = block_storage_volume_ids
        __props__.__dict__["created_at"] = created_at
        __props__.__dict__["deploy_target_id"] = deploy_target_id
        __props__.__dict__["destroy_protected"] = destroy_protected
        __props__.__dict__["disk_size"] = disk_size
        __props__.__dict__["elastic_ip_ids"] = elastic_ip_ids
        __props__.__dict__["ipv6"] = ipv6
        __props__.__dict__["ipv6_address"] = ipv6_address
        __props__.__dict__["labels"] = labels
        __props__.__dict__["mac_address"] = mac_address
        __props__.__dict__["name"] = name
        __props__.__dict__["network_interfaces"] = network_interfaces
        __props__.__dict__["private"] = private
        __props__.__dict__["private_network_ids"] = private_network_ids
        __props__.__dict__["public_ip_address"] = public_ip_address
        __props__.__dict__["reverse_dns"] = reverse_dns
        __props__.__dict__["security_group_ids"] = security_group_ids
        __props__.__dict__["ssh_key"] = ssh_key
        __props__.__dict__["state"] = state
        __props__.__dict__["template_id"] = template_id
        __props__.__dict__["type"] = type
        __props__.__dict__["user_data"] = user_data
        __props__.__dict__["zone"] = zone
        return ComputeInstance(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="antiAffinityGroupIds")
    def anti_affinity_group_ids(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        ❗ A list of exoscale*anti*affinity_group (IDs) to attach to the instance (may only be set at creation time).
        """
        return pulumi.get(self, "anti_affinity_group_ids")

    @property
    @pulumi.getter(name="blockStorageVolumeIds")
    def block_storage_volume_ids(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        A list of exoscale*block*storage_volume (ID) to attach to the instance.
        """
        return pulumi.get(self, "block_storage_volume_ids")

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[str]:
        """
        The instance creation date.
        """
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="deployTargetId")
    def deploy_target_id(self) -> pulumi.Output[Optional[str]]:
        """
        ❗ A deploy target ID.
        """
        return pulumi.get(self, "deploy_target_id")

    @property
    @pulumi.getter(name="destroyProtected")
    def destroy_protected(self) -> pulumi.Output[Optional[bool]]:
        """
        Mark the instance as protected, the Exoscale API will refuse to delete the instance until the protection is removed (boolean; default: `false`).
        """
        return pulumi.get(self, "destroy_protected")

    @property
    @pulumi.getter(name="diskSize")
    def disk_size(self) -> pulumi.Output[int]:
        """
        The instance disk size (GiB; at least `10`). Can not be decreased after creation. **WARNING**: updating this attribute stops/restarts the instance.
        """
        return pulumi.get(self, "disk_size")

    @property
    @pulumi.getter(name="elasticIpIds")
    def elastic_ip_ids(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        A list of exoscale*elastic*ip (IDs) to attach to the instance.
        """
        return pulumi.get(self, "elastic_ip_ids")

    @property
    @pulumi.getter
    def ipv6(self) -> pulumi.Output[Optional[bool]]:
        """
        Enable IPv6 on the instance (boolean; default: `false`).
        """
        return pulumi.get(self, "ipv6")

    @property
    @pulumi.getter(name="ipv6Address")
    def ipv6_address(self) -> pulumi.Output[str]:
        """
        The instance (main network interface) IPv6 address (if enabled).
        """
        return pulumi.get(self, "ipv6_address")

    @property
    @pulumi.getter
    def labels(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        A map of key/value labels.
        """
        return pulumi.get(self, "labels")

    @property
    @pulumi.getter(name="macAddress")
    def mac_address(self) -> pulumi.Output[str]:
        """
        MAC address
        """
        return pulumi.get(self, "mac_address")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The compute instance name.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="networkInterfaces")
    def network_interfaces(self) -> pulumi.Output[Optional[Sequence['outputs.ComputeInstanceNetworkInterface']]]:
        """
        Private network interfaces (may be specified multiple times). Structure is documented below.
        """
        return pulumi.get(self, "network_interfaces")

    @property
    @pulumi.getter
    def private(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether the instance is private (no public IP addresses; default: false)
        """
        return pulumi.get(self, "private")

    @property
    @pulumi.getter(name="privateNetworkIds")
    @_utilities.deprecated("""Use the network_interface block instead.""")
    def private_network_ids(self) -> pulumi.Output[Sequence[str]]:
        """
        A list of private networks (IDs) attached to the instance. Please use the `network_interface.*.network_id` argument instead.
        """
        return pulumi.get(self, "private_network_ids")

    @property
    @pulumi.getter(name="publicIpAddress")
    def public_ip_address(self) -> pulumi.Output[str]:
        """
        The instance (main network interface) IPv4 address.
        """
        return pulumi.get(self, "public_ip_address")

    @property
    @pulumi.getter(name="reverseDns")
    def reverse_dns(self) -> pulumi.Output[Optional[str]]:
        """
        Domain name for reverse DNS record.
        """
        return pulumi.get(self, "reverse_dns")

    @property
    @pulumi.getter(name="securityGroupIds")
    def security_group_ids(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        A list of exoscale*security*group (IDs) to attach to the instance.
        """
        return pulumi.get(self, "security_group_ids")

    @property
    @pulumi.getter(name="sshKey")
    def ssh_key(self) -> pulumi.Output[Optional[str]]:
        """
        The exoscale*ssh*key (name) to authorize in the instance (may only be set at creation time).
        """
        return pulumi.get(self, "ssh_key")

    @property
    @pulumi.getter
    def state(self) -> pulumi.Output[str]:
        """
        The instance state (`running` or `stopped`; default: `running`).
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> pulumi.Output[str]:
        """
        ❗ The get_template (ID) to use when creating the instance.
        """
        return pulumi.get(self, "template_id")

    @property
    @pulumi.getter
    def type(self) -> pulumi.Output[str]:
        """
        The instance type (`<family>.<size>`, e.g. `standard.medium`; use the [Exoscale CLI](https://github.com/exoscale/cli/) - `exo compute instance-type list` - for the list of available types). **WARNING**: updating this attribute stops/restarts the instance.
        """
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="userData")
    def user_data(self) -> pulumi.Output[Optional[str]]:
        """
        [cloud-init](https://cloudinit.readthedocs.io/) configuration.
        """
        return pulumi.get(self, "user_data")

    @property
    @pulumi.getter
    def zone(self) -> pulumi.Output[str]:
        """
        ❗ The Exoscale [Zone](https://www.exoscale.com/datacenters/) name.
        """
        return pulumi.get(self, "zone")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['SshKeyArgs', 'SshKey']

@pulumi.input_type
class SshKeyArgs:
    def __init__(__self__, *,
                 public_key: pulumi.Input[str],
                 name: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a SshKey resource.
        :param pulumi.Input[str] public_key: ❗ The SSH *public* key that will be authorized in compute instances.
        :param pulumi.Input[str] name: ❗ The SSH key name.
        """
        pulumi.set(__self__, "public_key", public_key)
        if name is not None:
            pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter(name="publicKey")
    def public_key(self) -> pulumi.Input[str]:
        """
        ❗ The SSH *public* key that will be authorized in compute instances.
        """
        return pulumi.get(self, "public_key")

    @public_key.setter
    def public_key(self, value: pulumi.Input[str]):
        pulumi.set(self, "public_key", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The SSH key name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)


@pulumi.input_type
class _SshKeyState:
    def __init__(__self__, *,
                 fingerprint: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 public_key: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering SshKey resources.
        :param pulumi.Input[str] fingerprint: The SSH key unique identifier.
        :param pulumi.Input[str] name: ❗ The SSH key name.
        :param pulumi.Input[str] public_key: ❗ The SSH *public* key that will be authorized in compute instances.
        """
        if fingerprint is not None:
            pulumi.set(__self__, "fingerprint", fingerprint)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if public_key is not None:
            pulumi.set(__self__, "public_key", public_key)

    @property
    @pulumi.getter
    def fingerprint(self) -> Optional[pulumi.Input[str]]:
        """
        The SSH key unique identifier.
        """
        return pulumi.get(self, "fingerprint")

    @fingerprint.setter
    def fingerprint(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "fingerprint", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The SSH key name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="publicKey")
    def public_key(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The SSH *public* key that will be authorized in compute instances.
        """
        return pulumi.get(self, "public_key")

    @public_key.setter
    def public_key(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "public_key", value)


class SshKey(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 public_key: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        ## Import

        An existing SSH key may be imported as a resource by `<name>`:

        ```sh
        $ pulumi import exoscale:index/sshKey:SshKey \\
        ```

          exoscale_ssh_key.my_ssh_key \\

          my-ssh-key

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] name: ❗ The SSH key name.
        :param pulumi.Input[str] public_key: ❗ The SSH *public* key that will be authorized in compute instances.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: SshKeyArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Import

        An existing SSH key may be imported as a resource by `<name>`:

        ```sh
        $ pulumi import exoscale:index/sshKey:SshKey \\
        ```

          exoscale_ssh_key.my_ssh_key \\

          my-ssh-key

        :param str resource_name: The name of the resource.
        :param SshKeyArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(SshKeyArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 public_key: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = SshKeyArgs.__new__(SshKeyArgs)

            __props__.__dict__["name"] = name
            if public_key is None and not opts.urn:
                raise TypeError("Missing required property 'public_key'")
            __props__.__dict__["public_key"] = public_key
            __props__.__dict__["fingerprint"] = None
        super(SshKey, __self__).__init__(
            'exoscale:index/sshKey:SshKey',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            fingerprint: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            public_key: Optional[pulumi.Input[str]] = None) -> 'SshKey':
        """
        Get an existing SshKey resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] fingerprint: The SSH key unique identifier.
        :param pulumi.Input[str] name: ❗ The SSH key name.
        :param pulumi.Input[str] public_key: ❗ The SSH *public* key that will be authorized in compute instances.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _SshKeyState.__new__(_SshKeyState)

        __props__.__dict__["fingerprint"] = fingerprint
        __props__.__dict__["name"] = name
        __props__.__dict__["public_key"] = public_key
        return SshKey(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def fingerprint(self) -> pulumi.Output[str]:
        """
        The SSH key unique identifier.
        """
        return pulumi.get(self, "fingerprint")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        ❗ The SSH key name.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="publicKey")
    def public_key(self) -> pulumi.Output[str]:
        """
        ❗ The SSH *public* key that will be authorized in compute instances.
        """
        return pulumi.get(self, "public_key")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetAntiAffinityGroupResult',
    'AwaitableGetAntiAffinityGroupResult',
    'get_anti_affinity_group',
    'get_anti_affinity_group_output',
]

@pulumi.output_type
class GetAntiAffinityGroupResult:
    """
    A collection of values returned by getAntiAffinityGroup.
    """
    def __init__(__self__, id=None, instances=None, name=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if instances and not isinstance(instances, list):
            raise TypeError("Expected argument 'instances' to be a list")
        pulumi.set(__self__, "instances", instances)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter
    def id(self) -> Optional[str]:
        """
        The anti-affinity group ID to match (conflicts with `name`).
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def instances(self) -> Sequence[str]:
        """
        The list of attached exoscale*compute*instance (IDs).
        """
        return pulumi.get(self, "instances")

    @property
    @pulumi.getter
    def name(self) -> Optional[str]:
        """
        The group name to match (conflicts with `id`).
        """
        return pulumi.get(self, "name")


class AwaitableGetAntiAffinityGroupResult(GetAntiAffinityGroupResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAntiAffinityGroupResult(
            id=self.id,
            instances=self.instances,
            name=self.name)


def get_anti_affinity_group(id: Optional[str] = None,
                            name: Optional[str] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAntiAffinityGroupResult:
    """
    Fetch Exoscale [Anti-Affinity Groups](https://community.exoscale.com/documentation/compute/anti-affinity-groups/) data.

    Corresponding resource: exoscale_anti_affinity_group.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_exoscale as exoscale

    my_anti_affinity_group = exoscale.get_anti_affinity_group(name="my-anti-affinity-group")
    pulumi.export("myAntiAffinityGroupId", my_anti_affinity_group.id)
    ```

    Please refer to the examples
    directory for complete configuration examples.


    :param str id: The anti-affinity group ID to match (conflicts with `name`).
    :param str name: The group name to match (conflicts with `id`).
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['name'] = name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('exoscale:index/getAntiAffinityGroup:getAntiAffinityGroup', __args__, opts=opts, typ=GetAntiAffinityGroupResult).value

    return AwaitableGetAntiAffinityGroupResult(
        id=pulumi.get(__ret__, 'id'),
        instances=pulumi.get(__ret__, 'instances'),
        name=pulumi.get(__ret__, 'name'))
def get_anti_affinity_group_output(id: Optional[pulumi.Input[Optional[str]]] = None,
                                   name: Optional[pulumi.Input[Optional[str]]] = None,
                                   opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetAntiAffinityGroupResult]:
    """
    Fetch Exoscale [Anti-Affinity Groups](https://community.exoscale.com/documentation/compute/anti-affinity-groups/) data.

    Corresponding resource: exoscale_anti_affinity_group.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_exoscale as exoscale

    my_anti_affinity_group = exoscale.get_anti_affinity_group(name="my-anti-affinity-group")
    pulumi.export("myAntiAffinityGroupId", my_anti_affinity_group.id)
    ```

    Please refer to the examples
    directory for complete configuration examples.


    :param str id: The anti-affinity group ID to match (conflicts with `name`).
    :param str name: The group name to match (conflicts with `id`).
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['name'] = name
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('exoscale:index/getAntiAffinityGroup:getAntiAffinityGroup', __args__, opts=opts, typ=GetAntiAffinityGroupResult)
    return __ret__.apply(lambda __response__: GetAntiAffinityGroupResult(
        id=pulumi.get(__response__, 'id'),
        instances=pulumi.get(__response__, 'instances'),
        name=pulumi.get(__response__, 'name')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['SecurityGroupRuleArgs', 'SecurityGroupRule']

@pulumi.input_type
class SecurityGroupRuleArgs:
    def __init__(__self__, *,
                 type: pulumi.Input[str],
                 cidr: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 end_port: Optional[pulumi.Input[int]] = None,
                 icmp_code: Optional[pulumi.Input[int]] = None,
                 icmp_type: Optional[pulumi.Input[int]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 public_security_group: Optional[pulumi.Input[str]] = None,
                 security_group: Optional[pulumi.Input[str]] = None,
                 security_group_id: Optional[pulumi.Input[str]] = None,
                 start_port: Optional[pulumi.Input[int]] = None,
                 user_security_group: Optional[pulumi.Input[str]] = None,
                 user_security_group_id: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a SecurityGroupRule resource.
        :param pulumi.Input[str] type: ❗ The traffic direction to match (`INGRESS` or `EGRESS`).
        :param pulumi.Input[str] cidr: ❗ An (`INGRESS`) source / (`EGRESS`) destination IP subnet (in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)) to match (conflicts with `public_security_group`/`user_security_group`/`user_security_group_id`).
        :param pulumi.Input[str] description: ❗ A free-form text describing the security group rule.
        :param pulumi.Input[int] end_port: ❗ A `TCP`/`UDP` port range to match.
        :param pulumi.Input[int] icmp_code: ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        :param pulumi.Input[int] icmp_type: ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        :param pulumi.Input[str] protocol: ❗ The network protocol to match (`TCP`, `UDP`, `ICMP`, `ICMPv6`, `AH`, `ESP`, `GRE`, `IPIP` or `ALL`)
        :param pulumi.Input[str] public_security_group: ❗ An (`INGRESS`) source / (`EGRESS`) destination public security group name to match (conflicts with `cidr`/`user_security_group`/`user_security_group_id`).
        :param pulumi.Input[str] security_group: ❗ The parent security group name. Please use the `security_group_id` argument along the exoscale*security*group data source instead.
        :param pulumi.Input[str] security_group_id: ❗ The parent exoscale*security*group ID.
        :param pulumi.Input[int] start_port: ❗ A `TCP`/`UDP` port range to match.
        :param pulumi.Input[str] user_security_group: ❗ An (`INGRESS`) source / (`EGRESS`) destination security group name to match (conflicts with `cidr`/`public_security_group`/`user_security_group_id`). Please use the `user_security_group_id` argument along the exoscale*security*group data source instead.
        :param pulumi.Input[str] user_security_group_id: ❗ An (`INGRESS`) source / (`EGRESS`) destination security group ID to match (conflicts with `cidr`/`public_security_group`/`user_security_group)`).
        """
        pulumi.set(__self__, "type", type)
        if cidr is not None:
            pulumi.set(__self__, "cidr", cidr)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if end_port is not None:
            pulumi.set(__self__, "end_port", end_port)
        if icmp_code is not None:
            pulumi.set(__self__, "icmp_code", icmp_code)
        if icmp_type is not None:
            pulumi.set(__self__, "icmp_type", icmp_type)
        if protocol is not None:
            pulumi.set(__self__, "protocol", protocol)
        if public_security_group is not None:
            pulumi.set(__self__, "public_security_group", public_security_group)
        if security_group is not None:
            warnings.warn("""Deprecated in favor of `security_group_id`""", DeprecationWarning)
            pulumi.log.warn("""security_group is deprecated: Deprecated in favor of `security_group_id`""")
        if security_group is not None:
            pulumi.set(__self__, "security_group", security_group)
        if security_group_id is not None:
            pulumi.set(__self__, "security_group_id", security_group_id)
        if start_port is not None:
            pulumi.set(__self__, "start_port", start_port)
        if user_security_group is not None:
            warnings.warn("""Deprecated in favor of `user_security_group_id`""", DeprecationWarning)
            pulumi.log.warn("""user_security_group is deprecated: Deprecated in favor of `user_security_group_id`""")
        if user_security_group is not None:
            pulumi.set(__self__, "user_security_group", user_security_group)
        if user_security_group_id is not None:
            pulumi.set(__self__, "user_security_group_id", user_security_group_id)

    @property
    @pulumi.getter
    def type(self) -> pulumi.Input[str]:
        """
        ❗ The traffic direction to match (`INGRESS` or `EGRESS`).
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: pulumi.Input[str]):
        pulumi.set(self, "type", value)

    @property
    @pulumi.getter
    def cidr(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination IP subnet (in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)) to match (conflicts with `public_security_group`/`user_security_group`/`user_security_group_id`).
        """
        return pulumi.get(self, "cidr")

    @cidr.setter
    def cidr(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "cidr", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ A free-form text describing the security group rule.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="endPort")
    def end_port(self) -> Optional[pulumi.Input[int]]:
        """
        ❗ A `TCP`/`UDP` port range to match.
        """
        return pulumi.get(self, "end_port")

    @end_port.setter
    def end_port(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "end_port", value)

    @property
    @pulumi.getter(name="icmpCode")
    def icmp_code(self) -> Optional[pulumi.Input[int]]:
        """
        ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        """
        return pulumi.get(self, "icmp_code")

    @icmp_code.setter
    def icmp_code(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "icmp_code", value)

    @property
    @pulumi.getter(name="icmpType")
    def icmp_type(self) -> Optional[pulumi.Input[int]]:
        """
        ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        """
        return pulumi.get(self, "icmp_type")

    @icmp_type.setter
    def icmp_type(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "icmp_type", value)

    @property
    @pulumi.getter
    def protocol(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The network protocol to match (`TCP`, `UDP`, `ICMP`, `ICMPv6`, `AH`, `ESP`, `GRE`, `IPIP` or `ALL`)
        """
        return pulumi.get(self, "protocol")

    @protocol.setter
    def protocol(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "protocol", value)

    @property
    @pulumi.getter(name="publicSecurityGroup")
    def public_security_group(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination public security group name to match (conflicts with `cidr`/`user_security_group`/`user_security_group_id`).
        """
        return pulumi.get(self, "public_security_group")

    @public_security_group.setter
    def public_security_group(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "public_security_group", value)

    @property
    @pulumi.getter(name="securityGroup")
    @_utilities.deprecated("""Deprecated in favor of `security_group_id`""")
    def security_group(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The parent security group name. Please use the `security_group_id` argument along the exoscale*security*group data source instead.
        """
        return pulumi.get(self, "security_group")

    @security_group.setter
    def security_group(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_group", value)

    @property
    @pulumi.getter(name="securityGroupId")
    def security_group_id(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The parent exoscale*security*group ID.
        """
        return pulumi.get(self, "security_group_id")

    @security_group_id.setter
    def security_group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_group_id", value)

    @property
    @pulumi.getter(name="startPort")
    def start_port(self) -> Optional[pulumi.Input[int]]:
        """
        ❗ A `TCP`/`UDP` port range to match.
        """
        return pulumi.get(self, "start_port")

    @start_port.setter
    def start_port(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "start_port", value)

    @property
    @pulumi.getter(name="userSecurityGroup")
    @_utilities.deprecated("""Deprecated in favor of `user_security_group_id`""")
    def user_security_group(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination security group name to match (conflicts with `cidr`/`public_security_group`/`user_security_group_id`). Please use the `user_security_group_id` argument along the exoscale*security*group data source instead.
        """
        return pulumi.get(self, "user_security_group")

    @user_security_group.setter
    def user_security_group(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_security_group", value)

    @property
    @pulumi.getter(name="userSecurityGroupId")
    def user_security_group_id(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination security group ID to match (conflicts with `cidr`/`public_security_group`/`user_security_group)`).
        """
        return pulumi.get(self, "user_security_group_id")

    @user_security_group_id.setter
    def user_security_group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_security_group_id", value)


@pulumi.input_type
class _SecurityGroupRuleState:
    def __init__(__self__, *,
                 cidr: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 end_port: Optional[pulumi.Input[int]] = None,
                 icmp_code: Optional[pulumi.Input[int]] = None,
                 icmp_type: Optional[pulumi.Input[int]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 public_security_group: Optional[pulumi.Input[str]] = None,
                 security_group: Optional[pulumi.Input[str]] = None,
                 security_group_id: Optional[pulumi.Input[str]] = None,
                 start_port: Optional[pulumi.Input[int]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 user_security_group: Optional[pulumi.Input[str]] = None,
                 user_security_group_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering SecurityGroupRule resources.
        :param pulumi.Input[str] cidr: ❗ An (`INGRESS`) source / (`EGRESS`) destination IP subnet (in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)) to match (conflicts with `public_security_group`/`user_security_group`/`user_security_group_id`).
        :param pulumi.Input[str] description: ❗ A free-form text describing the security group rule.
        :param pulumi.Input[int] end_port: ❗ A `TCP`/`UDP` port range to match.
        :param pulumi.Input[int] icmp_code: ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        :param pulumi.Input[int] icmp_type: ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        :param pulumi.Input[str] protocol: ❗ The network protocol to match (`TCP`, `UDP`, `ICMP`, `ICMPv6`, `AH`, `ESP`, `GRE`, `IPIP` or `ALL`)
        :param pulumi.Input[str] public_security_group: ❗ An (`INGRESS`) source / (`EGRESS`) destination public security group name to match (conflicts with `cidr`/`user_security_group`/`user_security_group_id`).
        :param pulumi.Input[str] security_group: ❗ The parent security group name. Please use the `security_group_id` argument along the exoscale*security*group data source instead.
        :param pulumi.Input[str] security_group_id: ❗ The parent exoscale*security*group ID.
        :param pulumi.Input[int] start_port: ❗ A `TCP`/`UDP` port range to match.
        :param pulumi.Input[str] type: ❗ The traffic direction to match (`INGRESS` or `EGRESS`).
        :param pulumi.Input[str] user_security_group: ❗ An (`INGRESS`) source / (`EGRESS`) destination security group name to match (conflicts with `cidr`/`public_security_group`/`user_security_group_id`). Please use the `user_security_group_id` argument along the exoscale*security*group data source instead.
        :param pulumi.Input[str] user_security_group_id: ❗ An (`INGRESS`) source / (`EGRESS`) destination security group ID to match (conflicts with `cidr`/`public_security_group`/`user_security_group)`).
        """
        if cidr is not None:
            pulumi.set(__self__, "cidr", cidr)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if end_port is not None:
            pulumi.set(__self__, "end_port", end_port)
        if icmp_code is not None:
            pulumi.set(__self__, "icmp_code", icmp_code)
        if icmp_type is not None:
            pulumi.set(__self__, "icmp_type", icmp_type)
        if protocol is not None:
            pulumi.set(__self__, "protocol", protocol)
        if public_security_group is not None:
            pulumi.set(__self__, "public_security_group", public_security_group)
        if security_group is not None:
            warnings.warn("""Deprecated in favor of `security_group_id`""", DeprecationWarning)
            pulumi.log.warn("""security_group is deprecated: Deprecated in favor of `security_group_id`""")
        if security_group is not None:
            pulumi.set(__self__, "security_group", security_group)
        if security_group_id is not None:
            pulumi.set(__self__, "security_group_id", security_group_id)
        if start_port is not None:
            pulumi.set(__self__, "start_port", start_port)
        if type is not None:
            pulumi.set(__self__, "type", type)
        if user_security_group is not None:
            warnings.warn("""Deprecated in favor of `user_security_group_id`""", DeprecationWarning)
            pulumi.log.warn("""user_security_group is deprecated: Deprecated in favor of `user_security_group_id`""")
        if user_security_group is not None:
            pulumi.set(__self__, "user_security_group", user_security_group)
        if user_security_group_id is not None:
            pulumi.set(__self__, "user_security_group_id", user_security_group_id)

    @property
    @pulumi.getter
    def cidr(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination IP subnet (in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)) to match (conflicts with `public_security_group`/`user_security_group`/`user_security_group_id`).
        """
        return pulumi.get(self, "cidr")

    @cidr.setter
    def cidr(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "cidr", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ A free-form text describing the security group rule.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="endPort")
    def end_port(self) -> Optional[pulumi.Input[int]]:
        """
        ❗ A `TCP`/`UDP` port range to match.
        """
        return pulumi.get(self, "end_port")

    @end_port.setter
    def end_port(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "end_port", value)

    @property
    @pulumi.getter(name="icmpCode")
    def icmp_code(self) -> Optional[pulumi.Input[int]]:
        """
        ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        """
        return pulumi.get(self, "icmp_code")

    @icmp_code.setter
    def icmp_code(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "icmp_code", value)

    @property
    @pulumi.getter(name="icmpType")
    def icmp_type(self) -> Optional[pulumi.Input[int]]:
        """
        ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        """
        return pulumi.get(self, "icmp_type")

    @icmp_type.setter
    def icmp_type(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "icmp_type", value)

    @property
    @pulumi.getter
    def protocol(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The network protocol to match (`TCP`, `UDP`, `ICMP`, `ICMPv6`, `AH`, `ESP`, `GRE`, `IPIP` or `ALL`)
        """
        return pulumi.get(self, "protocol")

    @protocol.setter
    def protocol(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "protocol", value)

    @property
    @pulumi.getter(name="publicSecurityGroup")
    def public_security_group(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination public security group name to match (conflicts with `cidr`/`user_security_group`/`user_security_group_id`).
        """
        return pulumi.get(self, "public_security_group")

    @public_security_group.setter
    def public_security_group(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "public_security_group", value)

    @property
    @pulumi.getter(name="securityGroup")
    @_utilities.deprecated("""Deprecated in favor of `security_group_id`""")
    def security_group(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The parent security group name. Please use the `security_group_id` argument along the exoscale*security*group data source instead.
        """
        return pulumi.get(self, "security_group")

    @security_group.setter
    def security_group(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_group", value)

    @property
    @pulumi.getter(name="securityGroupId")
    def security_group_id(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The parent exoscale*security*group ID.
        """
        return pulumi.get(self, "security_group_id")

    @security_group_id.setter
    def security_group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_group_id", value)

    @property
    @pulumi.getter(name="startPort")
    def start_port(self) -> Optional[pulumi.Input[int]]:
        """
        ❗ A `TCP`/`UDP` port range to match.
        """
        return pulumi.get(self, "start_port")

    @start_port.setter
    def start_port(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "start_port", value)

    @property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ The traffic direction to match (`INGRESS` or `EGRESS`).
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "type", value)

    @property
    @pulumi.getter(name="userSecurityGroup")
    @_utilities.deprecated("""Deprecated in favor of `user_security_group_id`""")
    def user_security_group(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination security group name to match (conflicts with `cidr`/`public_security_group`/`user_security_group_id`). Please use the `user_security_group_id` argument along the exoscale*security*group data source instead.
        """
        return pulumi.get(self, "user_security_group")

    @user_security_group.setter
    def user_security_group(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_security_group", value)

    @property
    @pulumi.getter(name="userSecurityGroupId")
    def user_security_group_id(self) -> Optional[pulumi.Input[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination security group ID to match (conflicts with `cidr`/`public_security_group`/`user_security_group)`).
        """
        return pulumi.get(self, "user_security_group_id")

    @user_security_group_id.setter
    def user_security_group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_security_group_id", value)


class SecurityGroupRule(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cidr: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 end_port: Optional[pulumi.Input[int]] = None,
                 icmp_code: Optional[pulumi.Input[int]] = None,
                 icmp_type: Optional[pulumi.Input[int]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 public_security_group: Optional[pulumi.Input[str]] = None,
                 security_group: Optional[pulumi.Input[str]] = None,
                 security_group_id: Optional[pulumi.Input[str]] = None,
                 start_port: Optional[pulumi.Input[int]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 user_security_group: Optional[pulumi.Input[str]] = None,
                 user_security_group_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manage Exoscale [Security Group](https://community.exoscale.com/documentation/compute/security-groups/) Rules.

        ## Example Usage

        ```python
        import pulumi
        import pulumiverse_exoscale as exoscale

        my_security_group = exoscale.SecurityGroup("mySecurityGroup")
        my_security_group_rule = exoscale.SecurityGroupRule("mySecurityGroupRule",
            security_group_id=my_security_group.id,
            type="INGRESS",
            protocol="TCP",
            cidr="0.0.0.0/0",
            start_port=80,
            end_port=80)
        ```

        Please refer to the examples
        directory for complete configuration examples.

        ## Import

        An existing security group rule may be imported by `<security-group-ID>/<security-group-rule-ID>`:

        ```sh
        $ pulumi import exoscale:index/securityGroupRule:SecurityGroupRule \\
        ```

          exoscale_security_group_rule.my_security_group_rule \\

          f81d4fae-7dec-11d0-a765-00a0c91e6bf6/9ecc6b8b-73d4-4211-8ced-f7f29bb79524

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] cidr: ❗ An (`INGRESS`) source / (`EGRESS`) destination IP subnet (in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)) to match (conflicts with `public_security_group`/`user_security_group`/`user_security_group_id`).
        :param pulumi.Input[str] description: ❗ A free-form text describing the security group rule.
        :param pulumi.Input[int] end_port: ❗ A `TCP`/`UDP` port range to match.
        :param pulumi.Input[int] icmp_code: ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        :param pulumi.Input[int] icmp_type: ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        :param pulumi.Input[str] protocol: ❗ The network protocol to match (`TCP`, `UDP`, `ICMP`, `ICMPv6`, `AH`, `ESP`, `GRE`, `IPIP` or `ALL`)
        :param pulumi.Input[str] public_security_group: ❗ An (`INGRESS`) source / (`EGRESS`) destination public security group name to match (conflicts with `cidr`/`user_security_group`/`user_security_group_id`).
        :param pulumi.Input[str] security_group: ❗ The parent security group name. Please use the `security_group_id` argument along the exoscale*security*group data source instead.
        :param pulumi.Input[str] security_group_id: ❗ The parent exoscale*security*group ID.
        :param pulumi.Input[int] start_port: ❗ A `TCP`/`UDP` port range to match.
        :param pulumi.Input[str] type: ❗ The traffic direction to match (`INGRESS` or `EGRESS`).
        :param pulumi.Input[str] user_security_group: ❗ An (`INGRESS`) source / (`EGRESS`) destination security group name to match (conflicts with `cidr`/`public_security_group`/`user_security_group_id`). Please use the `user_security_group_id` argument along the exoscale*security*group data source instead.
        :param pulumi.Input[str] user_security_group_id: ❗ An (`INGRESS`) source / (`EGRESS`) destination security group ID to match (conflicts with `cidr`/`public_security_group`/`user_security_group)`).
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: SecurityGroupRuleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manage Exoscale [Security Group](https://community.exoscale.com/documentation/compute/security-groups/) Rules.

        ## Example Usage

        ```python
        import pulumi
        import pulumiverse_exoscale as exoscale

        my_security_group = exoscale.SecurityGroup("mySecurityGroup")
        my_security_group_rule = exoscale.SecurityGroupRule("mySecurityGroupRule",
            security_group_id=my_security_group.id,
            type="INGRESS",
            protocol="TCP",
            cidr="0.0.0.0/0",
            start_port=80,
            end_port=80)
        ```

        Please refer to the examples
        directory for complete configuration examples.

        ## Import

        An existing security group rule may be imported by `<security-group-ID>/<security-group-rule-ID>`:

        ```sh
        $ pulumi import exoscale:index/securityGroupRule:SecurityGroupRule \\
        ```

          exoscale_security_group_rule.my_security_group_rule \\

          f81d4fae-7dec-11d0-a765-00a0c91e6bf6/9ecc6b8b-73d4-4211-8ced-f7f29bb79524

        :param str resource_name: The name of the resource.
        :param SecurityGroupRuleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(SecurityGroupRuleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cidr: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 end_port: Optional[pulumi.Input[int]] = None,
                 icmp_code: Optional[pulumi.Input[int]] = None,
                 icmp_type: Optional[pulumi.Input[int]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 public_security_group: Optional[pulumi.Input[str]] = None,
                 security_group: Optional[pulumi.Input[str]] = None,
                 security_group_id: Optional[pulumi.Input[str]] = None,
                 start_port: Optional[pulumi.Input[int]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 user_security_group: Optional[pulumi.Input[str]] = None,
                 user_security_group_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = SecurityGroupRuleArgs.__new__(SecurityGroupRuleArgs)

            __props__.__dict__["cidr"] = cidr
            __props__.__dict__["description"] = description
            __props__.__dict__["end_port"] = end_port
            __props__.__dict__["icmp_code"] = icmp_code
            __props__.__dict__["icmp_type"] = icmp_type
            __props__.__dict__["protocol"] = protocol
            __props__.__dict__["public_security_group"] = public_security_group
            __props__.__dict__["security_group"] = security_group
            __props__.__dict__["security_group_id"] = security_group_id
            __props__.__dict__["start_port"] = start_port
            if type is None and not opts.urn:
                raise TypeError("Missing required property 'type'")
            __props__.__dict__["type"] = type
            __props__.__dict__["user_security_group"] = user_security_group
            __props__.__dict__["user_security_group_id"] = user_security_group_id
        super(SecurityGroupRule, __self__).__init__(
            'exoscale:index/securityGroupRule:SecurityGroupRule',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            cidr: Optional[pulumi.Input[str]] = None,
            description: Optional[pulumi.Input[str]] = None,
            end_port: Optional[pulumi.Input[int]] = None,
            icmp_code: Optional[pulumi.Input[int]] = None,
            icmp_type: Optional[pulumi.Input[int]] = None,
            protocol: Optional[pulumi.Input[str]] = None,
            public_security_group: Optional[pulumi.Input[str]] = None,
            security_group: Optional[pulumi.Input[str]] = None,
            security_group_id: Optional[pulumi.Input[str]] = None,
            start_port: Optional[pulumi.Input[int]] = None,
            type: Optional[pulumi.Input[str]] = None,
            user_security_group: Optional[pulumi.Input[str]] = None,
            user_security_group_id: Optional[pulumi.Input[str]] = None) -> 'SecurityGroupRule':
        """
        Get an existing SecurityGroupRule resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] cidr: ❗ An (`INGRESS`) source / (`EGRESS`) destination IP subnet (in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)) to match (conflicts with `public_security_group`/`user_security_group`/`user_security_group_id`).
        :param pulumi.Input[str] description: ❗ A free-form text describing the security group rule.
        :param pulumi.Input[int] end_port: ❗ A `TCP`/`UDP` port range to match.
        :param pulumi.Input[int] icmp_code: ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        :param pulumi.Input[int] icmp_type: ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        :param pulumi.Input[str] protocol: ❗ The network protocol to match (`TCP`, `UDP`, `ICMP`, `ICMPv6`, `AH`, `ESP`, `GRE`, `IPIP` or `ALL`)
        :param pulumi.Input[str] public_security_group: ❗ An (`INGRESS`) source / (`EGRESS`) destination public security group name to match (conflicts with `cidr`/`user_security_group`/`user_security_group_id`).
        :param pulumi.Input[str] security_group: ❗ The parent security group name. Please use the `security_group_id` argument along the exoscale*security*group data source instead.
        :param pulumi.Input[str] security_group_id: ❗ The parent exoscale*security*group ID.
        :param pulumi.Input[int] start_port: ❗ A `TCP`/`UDP` port range to match.
        :param pulumi.Input[str] type: ❗ The traffic direction to match (`INGRESS` or `EGRESS`).
        :param pulumi.Input[str] user_security_group: ❗ An (`INGRESS`) source / (`EGRESS`) destination security group name to match (conflicts with `cidr`/`public_security_group`/`user_security_group_id`). Please use the `user_security_group_id` argument along the exoscale*security*group data source instead.
        :param pulumi.Input[str] user_security_group_id: ❗ An (`INGRESS`) source / (`EGRESS`) destination security group ID to match (conflicts with `cidr`/`public_security_group`/`user_security_group)`).
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _SecurityGroupRuleState.__new__(_SecurityGroupRuleState)

        __props__.__dict__["cidr"] = cidr
        __props__.__dict__["description"] = description
        __props__.__dict__["end_port"] = end_port
        __props__.__dict__["icmp_code"] = icmp_code
        __props__.__dict__["icmp_type"] = icmp_type
        __props__.__dict__["protocol"] = protocol
        __props__.__dict__["public_security_group"] = public_security_group
        __props__.__dict__["security_group"] = security_group
        __props__.__dict__["security_group_id"] = security_group_id
        __props__.__dict__["start_port"] = start_port
        __props__.__dict__["type"] = type
        __props__.__dict__["user_security_group"] = user_security_group
        __props__.__dict__["user_security_group_id"] = user_security_group_id
        return SecurityGroupRule(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def cidr(self) -> pulumi.Output[Optional[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination IP subnet (in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)) to match (conflicts with `public_security_group`/`user_security_group`/`user_security_group_id`).
        """
        return pulumi.get(self, "cidr")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        ❗ A free-form text describing the security group rule.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="endPort")
    def end_port(self) -> pulumi.Output[Optional[int]]:
        """
        ❗ A `TCP`/`UDP` port range to match.
        """
        return pulumi.get(self, "end_port")

    @property
    @pulumi.getter(name="icmpCode")
    def icmp_code(self) -> pulumi.Output[Optional[int]]:
        """
        ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        """
        return pulumi.get(self, "icmp_code")

    @property
    @pulumi.getter(name="icmpType")
    def icmp_type(self) -> pulumi.Output[Optional[int]]:
        """
        ❗ An ICMP/ICMPv6 [type/code](https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol#Control_messages) to match.
        """
        return pulumi.get(self, "icmp_type")

    @property
    @pulumi.getter
    def protocol(self) -> pulumi.Output[Optional[str]]:
        """
        ❗ The network protocol to match (`TCP`, `UDP`, `ICMP`, `ICMPv6`, `AH`, `ESP`, `GRE`, `IPIP` or `ALL`)
        """
        return pulumi.get(self, "protocol")

    @property
    @pulumi.getter(name="publicSecurityGroup")
    def public_security_group(self) -> pulumi.Output[str]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination public security group name to match (conflicts with `cidr`/`user_security_group`/`user_security_group_id`).
        """
        return pulumi.get(self, "public_security_group")

    @property
    @pulumi.getter(name="securityGroup")
    @_utilities.deprecated("""Deprecated in favor of `security_group_id`""")
    def security_group(self) -> pulumi.Output[str]:
        """
        ❗ The parent security group name. Please use the `security_group_id` argument along the exoscale*security*group data source instead.
        """
        return pulumi.get(self, "security_group")

    @property
    @pulumi.getter(name="securityGroupId")
    def security_group_id(self) -> pulumi.Output[str]:
        """
        ❗ The parent exoscale*security*group ID.
        """
        return pulumi.get(self, "security_group_id")

    @property
    @pulumi.getter(name="startPort")
    def start_port(self) -> pulumi.Output[Optional[int]]:
        """
        ❗ A `TCP`/`UDP` port range to match.
        """
        return pulumi.get(self, "start_port")

    @property
    @pulumi.getter
    def type(self) -> pulumi.Output[str]:
        """
        ❗ The traffic direction to match (`INGRESS` or `EGRESS`).
        """
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="userSecurityGroup")
    @_utilities.deprecated("""Deprecated in favor of `user_security_group_id`""")
    def user_security_group(self) -> pulumi.Output[str]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination security group name to match (conflicts with `cidr`/`public_security_group`/`user_security_group_id`). Please use the `user_security_group_id` argument along the exoscale*security*group data source instead.
        """
        return pulumi.get(self, "user_security_group")

    @property
    @pulumi.getter(name="userSecurityGroupId")
    def user_security_group_id(self) -> pulumi.Output[Optional[str]]:
        """
        ❗ An (`INGRESS`) source / (`EGRESS`) destination security group ID to match (conflicts with `cidr`/`public_security_group`/`user_security_group)`).
        """
        return pulumi.get(self, "user_security_group_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetElasticIpResult',
    'AwaitableGetElasticIpResult',
    'get_elastic_ip',
    'get_elastic_ip_output',
]

@pulumi.output_type
class GetElasticIpResult:
    """
    A collection of values returned by getElasticIp.
    """
    def __init__(__self__, address_family=None, cidr=None, description=None, healthchecks=None, id=None, ip_address=None, labels=None, reverse_dns=None, zone=None):
        if address_family and not isinstance(address_family, str):
            raise TypeError("Expected argument 'address_family' to be a str")
        pulumi.set(__self__, "address_family", address_family)
        if cidr and not isinstance(cidr, str):
            raise TypeError("Expected argument 'cidr' to be a str")
        pulumi.set(__self__, "cidr", cidr)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if healthchecks and not isinstance(healthchecks, list):
            raise TypeError("Expected argument 'healthchecks' to be a list")
        pulumi.set(__self__, "healthchecks", healthchecks)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ip_address and not isinstance(ip_address, str):
            raise TypeError("Expected argument 'ip_address' to be a str")
        pulumi.set(__self__, "ip_address", ip_address)
        if labels and not isinstance(labels, dict):
            raise TypeError("Expected argument 'labels' to be a dict")
        pulumi.set(__self__, "labels", labels)
        if reverse_dns and not isinstance(reverse_dns, str):
            raise TypeError("Expected argument 'reverse_dns' to be a str")
        pulumi.set(__self__, "reverse_dns", reverse_dns)
        if zone and not isinstance(zone, str):
            raise TypeError("Expected argument 'zone' to be a str")
        pulumi.set(__self__, "zone", zone)

    @property
    @pulumi.getter(name="addressFamily")
    def address_family(self) -> str:
        """
        The Elastic IP (EIP) address family (`inet4` or `inet6`).
        """
        return pulumi.get(self, "address_family")

    @property
    @pulumi.getter
    def cidr(self) -> str:
        """
        The Elastic IP (EIP) CIDR.
        """
        return pulumi.get(self, "cidr")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        The Elastic IP (EIP) description.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter
    def healthchecks(self) -> Sequence['outputs.GetElasticIpHealthcheckResult']:
        """
        The *managed* EIP healthcheck configuration.
        """
        return pulumi.get(self, "healthchecks")

    @property
    @pulumi.getter
    def id(self) -> Optional[str]:
        """
        The Elastic IP (EIP) ID to match (conflicts with `ip_address` and `labels`).
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="ipAddress")
    def ip_address(self) -> Optional[str]:
        """
        The EIP IPv4 or IPv6 address to match (conflicts with `id` and `labels`).
        """
        return pulumi.get(self, "ip_address")

    @property
    @pulumi.getter
    def labels(self) -> Optional[Mapping[str, str]]:
        """
        The EIP labels to match (conflicts with `ip_address` and `id`).
        """
        return pulumi.get(self, "labels")

    @property
    @pulumi.getter(name="reverseDns")
    def reverse_dns(self) -> str:
        """
        Domain name for reverse DNS record.
        """
        return pulumi.get(self, "reverse_dns")

    @property
    @pulumi.getter
    def zone(self) -> str:
        """
        The Exocale [Zone](https://www.exoscale.com/datacenters/) name.
        """
        return pulumi.get(self, "zone")


class AwaitableGetElasticIpResult(GetElasticIpResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetElasticIpResult(
            address_family=self.address_family,
            cidr=self.cidr,
            description=self.description,
            healthchecks=self.healthchecks,
            id=self.id,
            ip_address=self.ip_address,
            labels=self.labels,
            reverse_dns=self.reverse_dns,
            zone=self.zone)


def get_elastic_ip(id: Optional[str] = None,
                   ip_address: Optional[str] = None,
                   labels: Optional[Mapping[str, str]] = None,
                   zone: Optional[str] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetElasticIpResult:
    """
    Fetch Exoscale [Elastic IPs (EIP)](https://community.exoscale.com/documentation/compute/eip/) data.

    Corresponding resource: exoscale_elastic_ip.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_exoscale as exoscale

    my_elastic_ip = exoscale.get_elastic_ip(zone="ch-gva-2",
        ip_address="1.2.3.4")
    pulumi.export("myElasticIpId", my_elastic_ip.id)
    ```

    Please refer to the examples
    directory for complete configuration examples.


    :param str id: The Elastic IP (EIP) ID to match (conflicts with `ip_address` and `labels`).
    :param str ip_address: The EIP IPv4 or IPv6 address to match (conflicts with `id` and `labels`).
    :param Mapping[str, str] labels: The EIP labels to match (conflicts with `ip_address` and `id`).
    :param str zone: The Exocale [Zone](https://www.exoscale.com/datacenters/) name.
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['ipAddress'] = ip_address
    __args__['labels'] = labels
    __args__['zone'] = zone
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('exoscale:index/getElasticIp:getElasticIp', __args__, opts=opts, typ=GetElasticIpResult).value

    return AwaitableGetElasticIpResult(
        address_family=pulumi.get(__ret__, 'address_family'),
        cidr=pulumi.get(__ret__, 'cidr'),
        description=pulumi.get(__ret__, 'description'),
        healthchecks=pulumi.get(__ret__, 'healthchecks'),
        id=pulumi.get(__ret__, 'id'),
        ip_address=pulumi.get(__ret__, 'ip_address'),
        labels=pulumi.get(__ret__, 'labels'),
        reverse_dns=pulumi.get(__ret__, 'reverse_dns'),
        zone=pulumi.get(__ret__, 'zone'))
def get_elastic_ip_output(id: Optional[pulumi.Input[Optional[str]]] = None,
                          ip_address: Optional[pulumi.Input[Optional[str]]] = None,
                          labels: Optional[pulumi.Input[Optional[Mapping[str, str]]]] = None,
                          zone: Optional[pulumi.Input[str]] = None,
                          opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetElasticIpResult]:
    """
    Fetch Exoscale [Elastic IPs (EIP)](https://community.exoscale.com/documentation/compute/eip/) data.

    Corresponding resource: exoscale_elastic_ip.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_exoscale as exoscale

    my_elastic_ip = exoscale.get_elastic_ip(zone="ch-gva-2",
        ip_address="1.2.3.4")
    pulumi.export("myElasticIpId", my_elastic_ip.id)
    ```

    Please refer to the examples
    directory for complete configuration examples.


    :param str id: The Elastic IP (EIP) ID to match (conflicts with `ip_address` and `labels`).
    :param str ip_address: The EIP IPv4 or IPv6 address to match (conflicts with `id` and `labels`).
    :param Mapping[str, str] labels: The EIP labels to match (conflicts with `ip_address` and `id`).
    :param str zone: The Exocale [Zone](https://www.exoscale.com/datacenters/) name.
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['ipAddress'] = ip_address
    __args__['labels'] = labels
    __args__['zone'] = zone
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('exoscale:index/getElasticIp:getElasticIp', __args__, opts=opts, typ=GetElasticIpResult)
    return __ret__.apply(lambda __response__: GetElasticIpResult(
        address_family=pulumi.get(__response__, 'address_family'),
        cidr=pulumi.get(__response__, 'cidr'),
        description=pulumi.get(__response__, 'description'),
        healthchecks=pulumi.get(__response__, 'healthchecks'),
        id=pulumi.get(__response__, 'id'),
        ip_address=pulumi.get(__response__, 'ip_address'),
        labels=pulumi.get(__response__, 'labels'),
        reverse_dns=pulumi.get(__response__, 'reverse_dns'),
        zone=pulumi.get(__response__, 'zone')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from . import _utilities
import typing
# Export this package's modules as members:
from .anti_affinity_group import *
from .block_storage_volume import *
from .block_storage_volume_snapshot import *
from .compute_instance import *
from .database import *
from .domain import *
from .domain_record import *
from .elastic_ip import *
from .get_anti_affinity_group import *
from .get_block_storage_volume import *
from .get_block_storage_volume_snapshot import *
from .get_compute_instance import *
from .get_compute_instance_list import *
from .get_database_uri import *
from .get_domain import *
from .get_domain_record import *
from .get_elastic_ip import *
from .get_iam_api_key import *
from .get_iam_org_policy import *
from .get_iam_role import *
from .get_instance_pool import *
from .get_instance_pool_list import *
from .get_nlb import *
from .get_nlb_service_list import *
from .get_private_network import *
from .get_security_group import *
from .get_sks_cluster import *
from .get_sks_cluster_list import *
from .get_sks_nodepool import *
from .get_sks_nodepool_list import *
from .get_sos_bucket_policy import *
from .get_template import *
from .get_zones import *
from .iam_access_key import *
from .iam_api_key import *
from .iam_org_policy import *
from .iam_role import *
from .instance_pool import *
from .nlb import *
from .nlb_service import *
from .private_network import *
from .provider import *
from .security_group import *
from .security_group_rule import *
from .sks_cluster import *
from .sks_kubeconfig import *
from .sks_nodepool import *
from .sos_bucket_policy import *
from .ssh_key import *
from ._inputs import *
from . import outputs

# Make subpackages available:
if typing.TYPE_CHECKING:
    import pulumiverse_exoscale.config as __config
    config = __config
else:
    config = _utilities.lazy_import('pulumiverse_exoscale.config')

_utilities.register(
    resource_modules="""
[
 {
  "pkg": "exoscale",
  "mod": "index/antiAffinityGroup",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/antiAffinityGroup:AntiAffinityGroup": "AntiAffinityGroup"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/blockStorageVolume",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/blockStorageVolume:BlockStorageVolume": "BlockStorageVolume"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/blockStorageVolumeSnapshot",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/blockStorageVolumeSnapshot:BlockStorageVolumeSnapshot": "BlockStorageVolumeSnapshot"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/computeInstance",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/computeInstance:ComputeInstance": "ComputeInstance"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/database",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/database:Database": "Database"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/domain",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/domain:Domain": "Domain"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/domainRecord",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/domainRecord:DomainRecord": "DomainRecord"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/elasticIp",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/elasticIp:ElasticIp": "ElasticIp"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/iamAccessKey",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/iamAccessKey:IamAccessKey": "IamAccessKey"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/iamApiKey",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/iamApiKey:IamApiKey": "IamApiKey"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/iamOrgPolicy",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/iamOrgPolicy:IamOrgPolicy": "IamOrgPolicy"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/iamRole",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/iamRole:IamRole": "IamRole"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/instancePool",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/instancePool:InstancePool": "InstancePool"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/nlb",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/nlb:Nlb": "Nlb"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/nlbService",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/nlbService:NlbService": "NlbService"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/privateNetwork",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/privateNetwork:PrivateNetwork": "PrivateNetwork"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/securityGroup",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/securityGroup:SecurityGroup": "SecurityGroup"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/securityGroupRule",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/securityGroupRule:SecurityGroupRule": "SecurityGroupRule"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/sksCluster",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/sksCluster:SksCluster": "SksCluster"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/sksKubeconfig",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/sksKubeconfig:SksKubeconfig": "SksKubeconfig"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/sksNodepool",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/sksNodepool:SksNodepool": "SksNodepool"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/sosBucketPolicy",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/sosBucketPolicy:SosBucketPolicy": "SosBucketPolicy"
  }
 },
 {
  "pkg": "exoscale",
  "mod": "index/sshKey",
  "fqn": "pulumiverse_exoscale",
  "classes": {
   "exoscale:index/sshKey:SshKey": "SshKey"
  }
 }
]
""",
    resource_packages="""
[
 {
  "pkg": "exoscale",
  "token": "pulumi:providers:exoscale",
  "fqn": "pulumiverse_exoscale",
  "class": "Provider"
 }
]
"""
)



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['IamRoleArgs', 'IamRole']

@pulumi.input_type
class IamRoleArgs:
    def __init__(__self__, *,
                 description: Optional[pulumi.Input[str]] = None,
                 editable: Optional[pulumi.Input[bool]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 permissions: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 policy: Optional[pulumi.Input['IamRolePolicyArgs']] = None,
                 timeouts: Optional[pulumi.Input['IamRoleTimeoutsArgs']] = None):
        """
        The set of arguments for constructing a IamRole resource.
        :param pulumi.Input[str] description: A free-form text describing the IAM Role
        :param pulumi.Input[bool] editable: Defines if IAM Role Policy is editable or not.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: IAM Role labels.
        :param pulumi.Input[str] name: ❗Name of IAM Role.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] permissions: IAM Role permissions.
        :param pulumi.Input['IamRolePolicyArgs'] policy: IAM Policy.
        """
        if description is not None:
            pulumi.set(__self__, "description", description)
        if editable is not None:
            pulumi.set(__self__, "editable", editable)
        if labels is not None:
            pulumi.set(__self__, "labels", labels)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if permissions is not None:
            pulumi.set(__self__, "permissions", permissions)
        if policy is not None:
            pulumi.set(__self__, "policy", policy)
        if timeouts is not None:
            pulumi.set(__self__, "timeouts", timeouts)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        A free-form text describing the IAM Role
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def editable(self) -> Optional[pulumi.Input[bool]]:
        """
        Defines if IAM Role Policy is editable or not.
        """
        return pulumi.get(self, "editable")

    @editable.setter
    def editable(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "editable", value)

    @property
    @pulumi.getter
    def labels(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        IAM Role labels.
        """
        return pulumi.get(self, "labels")

    @labels.setter
    def labels(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "labels", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        ❗Name of IAM Role.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def permissions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        IAM Role permissions.
        """
        return pulumi.get(self, "permissions")

    @permissions.setter
    def permissions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "permissions", value)

    @property
    @pulumi.getter
    def policy(self) -> Optional[pulumi.Input['IamRolePolicyArgs']]:
        """
        IAM Policy.
        """
        return pulumi.get(self, "policy")

    @policy.setter
    def policy(self, value: Optional[pulumi.Input['IamRolePolicyArgs']]):
        pulumi.set(self, "policy", value)

    @property
    @pulumi.getter
    def timeouts(self) -> Optional[pulumi.Input['IamRoleTimeoutsArgs']]:
        return pulumi.get(self, "timeouts")

    @timeouts.setter
    def timeouts(self, value: Optional[pulumi.Input['IamRoleTimeoutsArgs']]):
        pulumi.set(self, "timeouts", value)


@pulumi.input_type
class _IamRoleState:
    def __init__(__self__, *,
                 description: Optional[pulumi.Input[str]] = None,
                 editable: Optional[pulumi.Input[bool]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 permissions: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 policy: Optional[pulumi.Input['IamRolePolicyArgs']] = None,
                 timeouts: Optional[pulumi.Input['IamRoleTimeoutsArgs']] = None):
        """
        Input properties used for looking up and filtering IamRole resources.
        :param pulumi.Input[str] description: A free-form text describing the IAM Role
        :param pulumi.Input[bool] editable: Defines if IAM Role Policy is editable or not.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: IAM Role labels.
        :param pulumi.Input[str] name: ❗Name of IAM Role.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] permissions: IAM Role permissions.
        :param pulumi.Input['IamRolePolicyArgs'] policy: IAM Policy.
        """
        if description is not None:
            pulumi.set(__self__, "description", description)
        if editable is not None:
            pulumi.set(__self__, "editable", editable)
        if labels is not None:
            pulumi.set(__self__, "labels", labels)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if permissions is not None:
            pulumi.set(__self__, "permissions", permissions)
        if policy is not None:
            pulumi.set(__self__, "policy", policy)
        if timeouts is not None:
            pulumi.set(__self__, "timeouts", timeouts)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        A free-form text describing the IAM Role
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def editable(self) -> Optional[pulumi.Input[bool]]:
        """
        Defines if IAM Role Policy is editable or not.
        """
        return pulumi.get(self, "editable")

    @editable.setter
    def editable(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "editable", value)

    @property
    @pulumi.getter
    def labels(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        IAM Role labels.
        """
        return pulumi.get(self, "labels")

    @labels.setter
    def labels(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "labels", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        ❗Name of IAM Role.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def permissions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        IAM Role permissions.
        """
        return pulumi.get(self, "permissions")

    @permissions.setter
    def permissions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "permissions", value)

    @property
    @pulumi.getter
    def policy(self) -> Optional[pulumi.Input['IamRolePolicyArgs']]:
        """
        IAM Policy.
        """
        return pulumi.get(self, "policy")

    @policy.setter
    def policy(self, value: Optional[pulumi.Input['IamRolePolicyArgs']]):
        pulumi.set(self, "policy", value)

    @property
    @pulumi.getter
    def timeouts(self) -> Optional[pulumi.Input['IamRoleTimeoutsArgs']]:
        return pulumi.get(self, "timeouts")

    @timeouts.setter
    def timeouts(self, value: Optional[pulumi.Input['IamRoleTimeoutsArgs']]):
        pulumi.set(self, "timeouts", value)


class IamRole(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 editable: Optional[pulumi.Input[bool]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 permissions: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 policy: Optional[pulumi.Input[Union['IamRolePolicyArgs', 'IamRolePolicyArgsDict']]] = None,
                 timeouts: Optional[pulumi.Input[Union['IamRoleTimeoutsArgs', 'IamRoleTimeoutsArgsDict']]] = None,
                 __props__=None):
        """
        Manage Exoscale [IAM](https://community.exoscale.com/documentation/iam/) Role.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: A free-form text describing the IAM Role
        :param pulumi.Input[bool] editable: Defines if IAM Role Policy is editable or not.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: IAM Role labels.
        :param pulumi.Input[str] name: ❗Name of IAM Role.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] permissions: IAM Role permissions.
        :param pulumi.Input[Union['IamRolePolicyArgs', 'IamRolePolicyArgsDict']] policy: IAM Policy.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[IamRoleArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manage Exoscale [IAM](https://community.exoscale.com/documentation/iam/) Role.

        :param str resource_name: The name of the resource.
        :param IamRoleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(IamRoleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 editable: Optional[pulumi.Input[bool]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 permissions: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 policy: Optional[pulumi.Input[Union['IamRolePolicyArgs', 'IamRolePolicyArgsDict']]] = None,
                 timeouts: Optional[pulumi.Input[Union['IamRoleTimeoutsArgs', 'IamRoleTimeoutsArgsDict']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = IamRoleArgs.__new__(IamRoleArgs)

            __props__.__dict__["description"] = description
            __props__.__dict__["editable"] = editable
            __props__.__dict__["labels"] = labels
            __props__.__dict__["name"] = name
            __props__.__dict__["permissions"] = permissions
            __props__.__dict__["policy"] = policy
            __props__.__dict__["timeouts"] = timeouts
        super(IamRole, __self__).__init__(
            'exoscale:index/iamRole:IamRole',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            description: Optional[pulumi.Input[str]] = None,
            editable: Optional[pulumi.Input[bool]] = None,
            labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            name: Optional[pulumi.Input[str]] = None,
            permissions: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            policy: Optional[pulumi.Input[Union['IamRolePolicyArgs', 'IamRolePolicyArgsDict']]] = None,
            timeouts: Optional[pulumi.Input[Union['IamRoleTimeoutsArgs', 'IamRoleTimeoutsArgsDict']]] = None) -> 'IamRole':
        """
        Get an existing IamRole resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: A free-form text describing the IAM Role
        :param pulumi.Input[bool] editable: Defines if IAM Role Policy is editable or not.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: IAM Role labels.
        :param pulumi.Input[str] name: ❗Name of IAM Role.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] permissions: IAM Role permissions.
        :param pulumi.Input[Union['IamRolePolicyArgs', 'IamRolePolicyArgsDict']] policy: IAM Policy.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _IamRoleState.__new__(_IamRoleState)

        __props__.__dict__["description"] = description
        __props__.__dict__["editable"] = editable
        __props__.__dict__["labels"] = labels
        __props__.__dict__["name"] = name
        __props__.__dict__["permissions"] = permissions
        __props__.__dict__["policy"] = policy
        __props__.__dict__["timeouts"] = timeouts
        return IamRole(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[str]:
        """
        A free-form text describing the IAM Role
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter
    def editable(self) -> pulumi.Output[bool]:
        """
        Defines if IAM Role Policy is editable or not.
        """
        return pulumi.get(self, "editable")

    @property
    @pulumi.getter
    def labels(self) -> pulumi.Output[Mapping[str, str]]:
        """
        IAM Role labels.
        """
        return pulumi.get(self, "labels")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        ❗Name of IAM Role.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def permissions(self) -> pulumi.Output[Sequence[str]]:
        """
        IAM Role permissions.
        """
        return pulumi.get(self, "permissions")

    @property
    @pulumi.getter
    def policy(self) -> pulumi.Output['outputs.IamRolePolicy']:
        """
        IAM Policy.
        """
        return pulumi.get(self, "policy")

    @property
    @pulumi.getter
    def timeouts(self) -> pulumi.Output[Optional['outputs.IamRoleTimeouts']]:
        return pulumi.get(self, "timeouts")

